from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

opto = SchLib(tool=SKIDL).add_parts(*[
        Part(name='4N25',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler Base Connected',description='DC Optocoupler, Phototransistor NPN, DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,aliases=['4N26', 'TIL111'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='6N135',dest=TEMPLATE,tool=SKIDL,keywords='High Speed Optocoupler',description='High Speed Optocoupler, TTL Compatible, CTR 24%, DIP8',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,aliases=['6N136'],pins=[
            Pin(num='1',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='A1050',dest=TEMPLATE,tool=SKIDL,keywords='light dependent photo resistor LDR',description='light dependent resistor',ref_prefix='R',num_units=1,fplist=['Resistor*LDR*D6.4*RM3.4*'],do_erc=True,aliases=['A1060'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='A9050',dest=TEMPLATE,tool=SKIDL,keywords='light dependent photo resistor LDR',description='light dependent resistor',ref_prefix='R',num_units=1,fplist=['Resistor*LDR*5x4.1*RM3*'],do_erc=True,aliases=['A9060', 'A9013'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='ACPL-C790',dest=TEMPLATE,tool=SKIDL,keywords='Isolation Amplifer',description='Precision Isolation Amplifer, ±0.5% Gain Tolerance, Bandwidth 200kHz, Stretched SO-8',ref_prefix='U',num_units=1,fplist=['SOIC*8_5.850x6.807mm*Pitch1.27mm*'],do_erc=True,aliases=['ACPL-C79A', 'ACPL-C79B'],pins=[
            Pin(num='1',name='VDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='IN+',do_erc=True),
            Pin(num='3',name='IN-',do_erc=True),
            Pin(num='4',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='OUT-',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='OUT+',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='VDD2',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ACPL-C870',dest=TEMPLATE,tool=SKIDL,keywords='Optycally Isolated Voltage Sensor',description='Precision Optycally Isolated Voltage Sensor, ±0.5% Gain Tolerance, Bandwidth 100kHz, Stretched SO-8',ref_prefix='U',num_units=1,fplist=['SOIC*8_5.850x6.807mm*Pitch1.27mm*'],do_erc=True,aliases=['ACPL-C87A', 'ACPL-C87B'],pins=[
            Pin(num='1',name='VDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='IN',do_erc=True),
            Pin(num='3',name='IN-',do_erc=True),
            Pin(num='4',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='OUT-',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='OUT+',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='VDD2',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ACPL-P343',dest=TEMPLATE,tool=SKIDL,keywords='MOSFET Driver IGBT Driver Optocoupler',description='Gate Drive Optocoupler, Output Current 4.0/4.0A, Propagation Delay 200ns, Stretched SO-6',ref_prefix='U',num_units=1,fplist=['SOIC-6_4.58x6.81mm_Pitch1.27mm'],do_erc=True,pins=[
            Pin(num='1',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='C',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='VEE',func=Pin.PWROUT,do_erc=True),
            Pin(num='5',name='VO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='VCC',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ASMT-YTC2-0AA02',dest=TEMPLATE,tool=SKIDL,description='Triple LED RVB (Avago Technology)',ref_prefix='U',num_units=1,fplist=['LED*RGB*PLCC*'],do_erc=True,pins=[
            Pin(num='1',name='KB',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='KG',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='KR',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='AR',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='AG',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='AB',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='BP103',dest=TEMPLATE,tool=SKIDL,keywords='NPN Phototransistor opto',description='NPN Phototransistor',ref_prefix='Q',num_units=1,fplist=['TO?18*Lens*'],do_erc=True,aliases=['BPY62'],pins=[
            Pin(num='1',name='E',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='B',do_erc=True),
            Pin(num='3',name='C',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='BP104-SMD',dest=TEMPLATE,tool=SKIDL,keywords='opto photodiode',description='Silicon Photodiode with V? Characteristic, SMD package',ref_prefix='D',num_units=1,fplist=['PhotoDiode*SMD*DIL2*4.5x4*'],do_erc=True,aliases=['BPW34-SMD', 'SFH2430', 'SFH2440'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='BPW21',dest=TEMPLATE,tool=SKIDL,keywords='photodiode opto',description='Silicon Photodiode for the visible spectral range',ref_prefix='D',num_units=1,fplist=['TO?5*Window*'],do_erc=True,aliases=['BPX61'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='BPW34',dest=TEMPLATE,tool=SKIDL,keywords='opto PIN photodiode',description='Silicon PIN Photodiode',ref_prefix='D',num_units=1,fplist=['PhotoDiode*SMD*DIL2*4.3x4.65*RM5.08*'],do_erc=True,aliases=['BP104'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='BPW40',dest=TEMPLATE,tool=SKIDL,keywords='NPN phototransistor',description='silicon NPN phototransistor',ref_prefix='Q',num_units=1,fplist=['LED*5mm*'],do_erc=True,aliases=['BP103B', 'BP103BF', 'SFH300'],pins=[
            Pin(num='1',name='C',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='E',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='BPW82',dest=TEMPLATE,tool=SKIDL,keywords='opto photo diode',description='Silicon PIN Photodiode with Daylight Blocking Filter',ref_prefix='D',num_units=1,fplist=['PhotoDiode*BPW82*'],do_erc=True,pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='BPX65',dest=TEMPLATE,tool=SKIDL,keywords='opto PIN photo diode',description='Silicon PIN Photodiode, TO-18 package',ref_prefix='D',num_units=1,fplist=['TO?18*Lens*'],do_erc=True,pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='CNY17-1',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler Base Connected',description='DC Optocoupler Base Connected, Vce 70V, CTR 160-320%, Viso 5000V (RMS), DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'DIP*W10.16mm*', 'DIP*W9.53mm*SMD*'],do_erc=True,aliases=['CNY17-2', 'CNY17-3', 'CNY17-4'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FOD420',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Random Phase',description='Random Phase Opto-Triac, Vdrm 800V, Ift 0.75mA, Itm 300mA, DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,aliases=['FOD4208', 'FOD4216', 'FOD4218'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FODM3011',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Random Phase Mini-Flat',description='Full Pitch Mini-Flat Random Phase Opto-Triac, Vdrm 600V, Ift 5mA, MFP 4L',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,aliases=['FODM3012', 'FODM3022', 'FODM3023', 'FODM3052', 'FODM3053'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='H11L1',dest=TEMPLATE,tool=SKIDL,keywords='High Speed Schmitt Optocoupler',description='Schmitt Trigger Output Optocoupler, High Speed, DIP-6, 5mA turn on threshold',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'DIP*W10.16mm*', 'DIP*W9.53mm*SMD*'],do_erc=True,aliases=['H11L2', 'H11L3'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='~',func=Pin.PWRIN,do_erc=True)]),
        Part(name='HCPL063',dest=TEMPLATE,tool=SKIDL,keywords='High Speed Optocoupler',description='Dual High Speed HCMOS/LSTTL/TTL Compatible Optocoupler, DIP-8, SOIC-8',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='A1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='C1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='A2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='C2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='VO2',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='7',name='VO1',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='8',name='VCC',func=Pin.PWRIN,do_erc=True)]),
        Part(name='HCPL2731',dest=TEMPLATE,tool=SKIDL,keywords='darlington optocoupler',description='Low Input Current high Gain Split Darlington Optocouplers',ref_prefix='U',num_units=2,fplist=['DIP*W7.62mm*', 'DIP*W9.53mm*SMD*'],do_erc=True,aliases=['HCPL2730'],pins=[
            Pin(num='8',name='VCC',func=Pin.PASSIVE,do_erc=True),
            Pin(num='1',name='ANODE',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='CATHODE',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='EMITTER',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='COLLECTOR',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='CATHODE',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='ANODE',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='EMITTER',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='COLLECTOR',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='HCPL3120',dest=TEMPLATE,tool=SKIDL,keywords='MOSFET Driver IGBT Driver Optocoupler',description='Gate Drive Optocoupler, Output Current 2.5/2.5A, PDIP-8',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='C',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='5',name='VEE',func=Pin.PWROUT,do_erc=True),
            Pin(num='6',name='VO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='VO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='VCC',func=Pin.PWRIN,do_erc=True)]),
        Part(name='IL300',dest=TEMPLATE,tool=SKIDL,keywords='Isolated Linear Photocoupler',description='Isolated Linear Photocoupler, High Gain, Wideband, DIP8/SMD8',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'DIP*W10.16mm*', 'DIP*W7.62mm*SMD*', 'DIP*W9.53mm*SMD*', 'DIP*W11.48mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='IRL81A',dest=TEMPLATE,tool=SKIDL,keywords='IR LED',description='850nm High Power Infrared Emitter, Side-Emitter package',ref_prefix='D',num_units=1,fplist=['LED*SideEmitter*Rectangular*W4.5mm*H1.6mm*'],do_erc=True,pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='IS471F',dest=TEMPLATE,tool=SKIDL,keywords='opto IR receiver',description='Photo Modules for PCM Remote Control Systems',ref_prefix='U',num_units=1,fplist=['OPIC*Sharp*IS471F*'],do_erc=True,pins=[
            Pin(num='1',name='Vs',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='GLo',func=Pin.OPENCOLL,do_erc=True)]),
        Part(name='IS485',dest=TEMPLATE,tool=SKIDL,keywords='opto receiver amplifier light detector OPIC',description='Bult-in Amplifier Type OPIC Light Detector',ref_prefix='U',num_units=1,fplist=['OPIC*Sharp*IS485*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='Vs',func=Pin.PWRIN,do_erc=True)]),
        Part(name='IS486',dest=TEMPLATE,tool=SKIDL,keywords='opto receiver amplifier light detector OPIC',description='Bult-in Amplifier Type OPIC Light Detector',ref_prefix='U',num_units=1,fplist=['OPIC*Sharp*IS485*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='Vs',func=Pin.PWRIN,do_erc=True)]),
        Part(name='KPS-5130',dest=TEMPLATE,tool=SKIDL,keywords='opto photodiode RGB color sensor',description='RGB Color Sensor',ref_prefix='PD',num_units=1,fplist=['KPS*5130*'],do_erc=True,pins=[
            Pin(num='1',name='AR',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='AG',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='AB',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='C',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LD271',dest=TEMPLATE,tool=SKIDL,keywords='IR LED',description='950nm High-Power IR-LED, 5mm',ref_prefix='D',num_units=1,fplist=['LED*5mm*'],do_erc=True,aliases=['SFH4546', 'CQY99', 'LD274', 'SFH4550'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LDR03',dest=TEMPLATE,tool=SKIDL,keywords='light dependent photo resistor LDR',description='light dependent resistor',ref_prefix='R',num_units=1,fplist=['Resistor*LDR*10x8.5*RM7.6*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LDR07',dest=TEMPLATE,tool=SKIDL,keywords='light dependent photo resistor LDR',description='light dependent resistor',ref_prefix='R',num_units=1,fplist=['Resistor*LDR*5.1x4.3*RM3.4*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LPT80A',dest=TEMPLATE,tool=SKIDL,keywords='NPN phototransistor',description='NPN phototransistor, sielooker package',ref_prefix='Q',num_units=1,fplist=['PhotoTransistor*SideLooker*'],do_erc=True,pins=[
            Pin(num='1',name='C',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='E',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-352T',dest=TEMPLATE,tool=SKIDL,keywords='NPN Darlington DC Optocoupler',description='DC Darlington Optocoupler, Vce 300V, CTR 1000%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-354T',dest=TEMPLATE,tool=SKIDL,keywords='NPN AC DC Optocoupler',description='AC/DC Optocoupler, Vce 35V, CTR 20%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-355T',dest=TEMPLATE,tool=SKIDL,keywords='NPN Darlington DC Optocoupler',description='DC Darlington Optocoupler, Vce 35V, CTR 600%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-356T',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler',description='DC Optocoupler, Vce 120V, CTR 50%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,aliases=['LTV-357T', 'LTV-358T'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-814',dest=TEMPLATE,tool=SKIDL,keywords='NPN AC DC Optocoupler',description='AC/DC Optocoupler, Vce 35V, CTR 20%, DIP4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-817',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler',description='DC Optocoupler, Vce 35V, CTR 50%, DIP-4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-824',dest=TEMPLATE,tool=SKIDL,keywords='Dual NPN AC DC Optocoupler',description='Dual AC/DC Optocoupler, Vce 35V, CTR 20%, DIP8',ref_prefix='U',num_units=2,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-827',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Dual Optocoupler',description='Dual DC Optocoupler, Vce 35V, CTR 50%, DIP-8',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-844',dest=TEMPLATE,tool=SKIDL,keywords='Quad NPN AC DC Optocoupler',description='Quad AC/DC Optocoupler, Vce 35V, CTR 20%, DIP16',ref_prefix='U',num_units=4,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='LTV-847',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Quad Optocoupler',description='Quad DC Optocoupler, Vce 35V, CTR 50%, DIP-16',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='M9960',dest=TEMPLATE,tool=SKIDL,keywords='light dependent photo resistor LDR',description='light dependent resistor',ref_prefix='R',num_units=1,fplist=['Resistor*LDR*5.2x5.2*RM3.5*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='MOC3010M',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Random Phase',description='Random Phase Opto-Triac, Vdrm 400V, Ift 5mA, DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'SMD-6*'],do_erc=True,aliases=['MOC3011M', 'MOC3012M', 'MOC3020M', 'MOC3021M', 'MOC3022M', 'MOC3023M'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='MOC3031M',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Zero Cross',description='Zero Cross Opto-Triac, Vdrm 400V, Ift 5mA, DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'SMD-6*'],do_erc=True,aliases=['MOC3032M', 'MOC3033M', 'MOC3041M', 'MOC3042M', 'MOC3043M'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='MOC3051M',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Random Phase',description='Random Phase Opto-Triac, Vdrm 600V, Ift 10mA, DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'SMD-6*'],do_erc=True,aliases=['MOC3052M'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='MOC3061M',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Zero Cross',description='Zero Cross Opto-Triac, Vdrm 600V, Ift 5mA, dv/dt 1000, DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'SMD-6*'],do_erc=True,aliases=['MOC3062M', 'MOC3063M', 'MOC3162M', 'MOC3163M'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='MOC3081M',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Zero Cross',description='Zero Cross Opto-Triac, Vdrm 800V, Ift 5mA, DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'SMD-6*'],do_erc=True,aliases=['MOC3082M', 'MOC3083M'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='OPTO-DARLINGTON',dest=TEMPLATE,tool=SKIDL,keywords='NPN Darlington DC Optocoupler',description='Generic DC Darlington Optocoupler',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='OPTO-THYRISTOR',dest=TEMPLATE,tool=SKIDL,keywords='OptoThyristor DC Optocoupler',description='Generic DC Optocoupler Thyristor Based',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='OPTO-TRANSISTOR',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler Base Connected',description='Generic Optocoupler with Base Connected',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='OPTO-TRANSISTOR-4',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Quad Optocoupler',description='Generic DC Quad Optocoupler',ref_prefix='U',num_units=4,do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='OPTO-TRIAC',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Random Phase',description='Generic Random Phase Opto-Triac',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='PC3H4',dest=TEMPLATE,tool=SKIDL,keywords='NPN AC DC Optocoupler',description='AC/DC Optocoupler, Vce 70V, CTR 100-250%, SOP4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,aliases=['PC3H4A'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='PC817',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler',description='DC Optocoupler, Vce 35V, CTR 50-300%, DIP4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='PC827',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Dual Optocoupler',description='DC Dual Optocoupler, Vce 35V, CTR 50-300%, DIP8',ref_prefix='U',num_units=2,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='PC837',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Triple Optocoupler',description='DC Triple Optocoupler, Vce 35V, CTR 50-300%, DIP12',ref_prefix='U',num_units=3,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='PC847',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Quad Optocoupler',description='DC Quad Optocoupler, Vce 35V, CTR 50-300%, DIP16',ref_prefix='U',num_units=4,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='PL450B',dest=TEMPLATE,tool=SKIDL,keywords='opto laserdiode',description='Green Laser Diode',ref_prefix='LD',num_units=1,fplist=['LaserDiode*TO38ICut*'],do_erc=True,aliases=['PL520', 'PLT5_510'],pins=[
            Pin(num='1',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='C',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='PLT5_488',dest=TEMPLATE,tool=SKIDL,keywords='opto laserdiode',description='Cyan Laser Diode',ref_prefix='LD',num_units=1,fplist=['LaserDiode*TO56*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='S102S01',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Random Phase Solid State Relays',description='Random Phase Opto-Triac, Vdrm 600V, Ift 8mA, IT 16A',ref_prefix='U',num_units=1,fplist=['SIP4*Sharp*SSR*Pitch7.62mm*'],do_erc=True,aliases=['S202S01', 'S116S01', 'S216S01', 'S112S01', 'S212S01'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='S102S02',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Zero Cross Solid State Relays',description='Zero Cross Opto-Triac, Vdrm 600V, Ift 8mA, IT 16A',ref_prefix='U',num_units=1,fplist=['SIP4*Sharp*SSR*Pitch7.62mm*'],do_erc=True,aliases=['S202S02', 'S116S02', 'S216S02'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH203',dest=TEMPLATE,tool=SKIDL,keywords='PIN photodiode IR',description='Silicon PIN Photodiode with Daylight Blocking Filter',ref_prefix='D',num_units=1,fplist=['LED*5mm*'],do_erc=True,aliases=['SFH203FA'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH205F',dest=TEMPLATE,tool=SKIDL,keywords='PIN Photodiode',description='Silicon PIN Photodiode with Daylight Blocking Filter',ref_prefix='D',num_units=1,fplist=['PhotoDiode*SFH205*'],do_erc=True,aliases=['SFH205FA'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH206K',dest=TEMPLATE,tool=SKIDL,keywords='opto PIN photodiode',description='Silicon PIN Photodiode',ref_prefix='D',num_units=1,fplist=['PhotoDiode*SFH205*'],do_erc=True,pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH216',dest=TEMPLATE,tool=SKIDL,keywords='opto PIN photo diode',description='Silicon PIN Photodiode With Very Short Switching Time, TO-18 package',ref_prefix='D',num_units=1,fplist=['TO?18*Window*'],do_erc=True,pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH225FA',dest=TEMPLATE,tool=SKIDL,keywords='opto PIN photodiode IR',description='Silicon PIN Photodiode with Daylight Blocking Filter',ref_prefix='D',num_units=1,fplist=['PhotoDiode*SFH225*'],do_erc=True,aliases=['SFH235FA'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH2400',dest=TEMPLATE,tool=SKIDL,keywords='opto photodiode',description='Silicon PIN Photodiode, SMD package',ref_prefix='D',num_units=1,fplist=['PhotoDiode*SMD*SmartDIL*'],do_erc=True,pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH2701',dest=TEMPLATE,tool=SKIDL,keywords='opto photodiode',description='High Speed PIN Photodiode, SMD package',ref_prefix='D',num_units=1,fplist=['LED*1206*'],do_erc=True,pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH309',dest=TEMPLATE,tool=SKIDL,keywords='npn phototransistor',description='Phototransistor NPN',ref_prefix='Q',num_units=1,fplist=['LED*3mm*'],do_erc=True,aliases=['BPW42'],pins=[
            Pin(num='1',name='C',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='E',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH4356P',dest=TEMPLATE,tool=SKIDL,keywords='opto IR LED',description='Infrared LED , 3mm LED package',ref_prefix='D',num_units=1,fplist=['LED*3mm*'],do_erc=True,aliases=['IR204A', 'TSAL4400', 'SFH4346'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH460',dest=TEMPLATE,tool=SKIDL,keywords='opto IR LED',description='GaAlAs Infrared LED, TO-18 package',ref_prefix='D',num_units=1,fplist=['TO?18*Window*'],do_erc=True,pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SFH482',dest=TEMPLATE,tool=SKIDL,keywords='opto IR LED',description='GaAlAs Infrared LED (880 nm), TO-18 package',ref_prefix='D',num_units=1,fplist=['TO?18*Window*'],do_erc=True,aliases=['SFH480'],pins=[
            Pin(num='1',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SPL_PL90',dest=TEMPLATE,tool=SKIDL,keywords='opto laserdiode',description='Pulsed Laser Diode in Plastic Package 25 W Peak Power',ref_prefix='LD',num_units=1,fplist=['LED*5mm*'],do_erc=True,pins=[
            Pin(num='1',name='C',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP127',dest=TEMPLATE,tool=SKIDL,keywords='NPN Darlington DC Optocoupler',description='DC Darlington Optocoupler, Vce 300V, CTR 1000%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP130',dest=TEMPLATE,tool=SKIDL,keywords='NPN AC DC Optocoupler Base Connected',description='AC DC Optocoupler Base Connected, Vce 80V, CTR 50-100%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP131',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler Base Connected',description='DC Optocoupler Base Connected, Vce 80V, CTR 50-200%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP137',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler Base Connected',description='DC Optocoupler Base Connected, Vce 80V, CTR 100-200%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP141G',dest=TEMPLATE,tool=SKIDL,keywords='OptoThyristor DC Optocoupler',description='DC Optocoupler Thyristor Based, Vdrm 400V, It 150mA, Viso 2500V, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP160G',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Random',description='Random Phase Opto-Triac, Vdrm 600V, Ift 7mA, SOP6',ref_prefix='U',num_units=1,do_erc=True,aliases=['TLP160J'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP161G',dest=TEMPLATE,tool=SKIDL,keywords='Opto-Triac Opto Triac Zero Cross',description='Zero Cross Opto-Triac, Vdrm 600V, Ift 7mA, SOP6',ref_prefix='U',num_units=1,do_erc=True,aliases=['TLP161J'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP175A',dest=TEMPLATE,tool=SKIDL,keywords='MOSFET Output Photorelay 1-Form-A',description='MOSFET Photorelay 1-Form-A, Voff 60V, Ion 100mA, SOP6',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP184',dest=TEMPLATE,tool=SKIDL,keywords='NPN AC DC Optocoupler',description='AC/DC Optocoupler, Vce 80V, CTR 50-100%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP185',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler',description='DC Optocoupler, Vce 80V, CTR 50-100%, MFSOP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP222A',dest=TEMPLATE,tool=SKIDL,keywords='MOSFET Output Photorelay 1-Form-A',description='MOSFET Photorelay 1-Form-A, Voff 60V, Ion 0,5A, DIP4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP222A-2',dest=TEMPLATE,tool=SKIDL,keywords='Dual MOSFET Output Photorelay 1-Form-A',description='Dual MOSFET Photorelay 1-Form-A, Voff 60V, Ion 0,5A, DIP8',ref_prefix='U',num_units=2,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP250',dest=TEMPLATE,tool=SKIDL,keywords='MOSFET Driver IGBT Driver Optocoupler',description='Gate Drive Optocoupler, Output Current 1.5/1.5A',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='2',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='C',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='5',name='VEE',func=Pin.PWROUT,do_erc=True),
            Pin(num='6',name='VO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='VO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='VCC',func=Pin.PWRIN,do_erc=True)]),
        Part(name='TLP290',dest=TEMPLATE,tool=SKIDL,keywords='NPN AC DC Phototransistor Optocoupler',description='AC/DC Phototransistor Optocoupler, Vce 80V, CTR 50-600%, SOP4',ref_prefix='U',num_units=1,fplist=['SOP*4*'],do_erc=True,aliases=['TCMT1600'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP290-4',dest=TEMPLATE,tool=SKIDL,keywords='NPN AC DC Quad Phototransistor Optocoupler',description='Quad AC/DC Phototransistor Optocoupler, Vce 80V, CTR 50-600%, SOP16',ref_prefix='U',num_units=4,fplist=['SOP*16*'],do_erc=True,aliases=['TCMT4600', 'TCMT4606'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP291',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler',description='DC Optocoupler, Vce 80V, CTR 50-100%, SOP4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP291-4',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Quad Optocoupler',description='Quad DC Optocoupler, Vce 80V, CTR 50-100%, SOP16',ref_prefix='U',num_units=4,fplist=['DIP*W7.62mm*SMD*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP3021',dest=TEMPLATE,tool=SKIDL,keywords='Photo-Triac Opto Triac',description='Photo-Triac, Vdrm 400V, Ift 5mA, DIP6',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,aliases=['TLP3022', 'TLP3023'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP627',dest=TEMPLATE,tool=SKIDL,keywords='NPN Darlington DC Optocoupler',description='DC Darlington Optocoupler, Vce 300V, CTR 1000%, DIP4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP627-2',dest=TEMPLATE,tool=SKIDL,keywords='NPN Dual Darlington DC Optocoupler',description='DC Dual Darlington Optocoupler, Vce 300V, CTR 1000%, DIP8',ref_prefix='U',num_units=2,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP627-4',dest=TEMPLATE,tool=SKIDL,keywords='NPN Quad Darlington DC Optocoupler',description='DC Quad Darlington Optocoupler, Vce 300V, CTR 1000%, DIP16',ref_prefix='U',num_units=4,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP785',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler',description='DC Optocoupler, Vce 80V, CTR 50-200%, DIP4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLP785F',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler',description='DC Optocoupler, Vce 80V, CTR 50-200%, DIP4F',ref_prefix='U',num_units=1,fplist=['DIP*W10.16mm*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TSDP341xx',dest=TEMPLATE,tool=SKIDL,keywords='opto IR receiver',description='IR Receiver Modules for Remote Control Systems',ref_prefix='U',num_units=1,fplist=['IRReceiver?Vishay?MOLD*'],do_erc=True,aliases=['TSDP343xx', 'TSOP341xx', 'TSOP343xx', 'TSOP345xx', 'TSOP41xx', 'TSOP43xx', 'TSOP45xx'],pins=[
            Pin(num='1',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='Vs',func=Pin.PWRIN,do_erc=True)]),
        Part(name='TSL2550-SO8',dest=TEMPLATE,tool=SKIDL,keywords='opto ambient light sensor',description='Ambient Light Sensor with SMbus Interface, SO-8 interface',ref_prefix='U',num_units=1,fplist=['SOIC*3.9x4.9mm*Pitch1.27mm*'],do_erc=True,pins=[
            Pin(num='1',name='Vdd',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='CLK',do_erc=True),
            Pin(num='6',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='7',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='8',name='DAT',func=Pin.BIDIR,do_erc=True)]),
        Part(name='TSL2550-T4',dest=TEMPLATE,tool=SKIDL,keywords='opto ambient light sensor',description='Ambient Light Sensor with SMbus Interface, T-4 interface',ref_prefix='U',num_units=1,fplist=['TSSL2550*'],do_erc=True,pins=[
            Pin(num='1',name='Vdd',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='CLK',do_erc=True),
            Pin(num='4',name='DAT',func=Pin.BIDIR,do_erc=True)]),
        Part(name='TSMP58138',dest=TEMPLATE,tool=SKIDL,keywords='opto IR repeater receiver',description='Photo Module (Repeater) for PCM Remote Control Systems',ref_prefix='U',num_units=1,fplist=['IRReceiver?Vishay?MINICAST*'],do_erc=True,aliases=['TSMP58000'],pins=[
            Pin(num='1',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='Vs',func=Pin.PWRIN,do_erc=True)]),
        Part(name='TSOP17xx',dest=TEMPLATE,tool=SKIDL,keywords='opto IR receiver',description='Photo Modules for PCM Remote Control Systems',ref_prefix='U',num_units=1,fplist=['IRReceiver?Vishay?CAST*'],do_erc=True,aliases=['TSOP312xx', 'TSOP314xx'],pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='Vs',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='OUT',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='TSOP32S40F',dest=TEMPLATE,tool=SKIDL,keywords='opto IR receiver',description='IR Receiver Modules for Remote Control Systems',ref_prefix='U',num_units=1,fplist=['IRReceiver?Vishay?MOLD*'],do_erc=True,aliases=['TSOP321xx', 'TSOP323xx', 'TSOP325xx', 'TSOP21xx', 'TSOP23xx', 'TSOP25xx'],pins=[
            Pin(num='1',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='Vs',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='TSOP331xx',dest=TEMPLATE,tool=SKIDL,keywords='opto IR receiver',description='IR Receiver Modules for Remote Control Systems',ref_prefix='U',num_units=1,fplist=['IRReceiver?Vishay?MINIMOLD*'],do_erc=True,aliases=['TSOP333xx', 'TSOP335xx', 'TSOP531xx', 'TSOP533xx', 'TSOP535xx'],pins=[
            Pin(num='1',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='Vs',func=Pin.PWRIN,do_erc=True)]),
        Part(name='TSOP34S40F',dest=TEMPLATE,tool=SKIDL,keywords='opto IR receiver',description='IR Receiver Modules for Remote Control Systems',ref_prefix='U',num_units=1,fplist=['IRReceiver?Vishay?MOLD*'],do_erc=True,pins=[
            Pin(num='1',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='Vs',func=Pin.PWRIN,do_erc=True)]),
        Part(name='TSOP581xx',dest=TEMPLATE,tool=SKIDL,keywords='opto IR receiver proximity sensor',description='IR Detector for Mid Range Proximity Sensor',ref_prefix='U',num_units=1,fplist=['IRReceiver?Vishay?MINICAST*'],do_erc=True,aliases=['TSOP583xx', 'TSOP585xx', 'TSOP382xx', 'TSOP384xx', 'TSOP582xx', 'TSOP584xx', 'TSOP38G36', 'TSSP58038', 'TSSP58038SS1XB', 'TSSP58P38'],pins=[
            Pin(num='1',name='OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='Vs',func=Pin.PWRIN,do_erc=True)]),
        Part(name='VO615A',dest=TEMPLATE,tool=SKIDL,keywords='NPN DC Optocoupler',description='DC Optocoupler, Vce 70V, CTR 200-400% @ 5mA, Viso 5000Vrms, DIP4',ref_prefix='U',num_units=1,fplist=['DIP*W7.62mm*', 'DIP*W10.16mm*', 'DIP*W7.62mm*SMD*', 'DIP*W9.53mm*SMD*', 'DIP*W11.48mm*SMD*'],do_erc=True,aliases=['VO615A-1', 'VO615A-2', 'VO615A-3', 'VO615A-4', 'VO615A-5', 'VO615A-6', 'VO615A-7', 'VO615A-8', 'VO615A-9'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='VT93xx',dest=TEMPLATE,tool=SKIDL,keywords='light dependent photo resistor LDR',description='light dependent resistor',ref_prefix='R',num_units=1,fplist=['Resistor*LDR*4.9x4.2*RM2.54*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

pspice = SchLib(tool=SKIDL).add_parts(*[
        Part(name='0',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='CAP',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['C']),
        Part(name='DIODE',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='INDUCTOR',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='ISOURCE',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='QNPN',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='QPNP',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='R',dest=TEMPLATE,tool=SKIDL,keywords='R DEV',description='Resistance',ref_prefix='R',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='VSOURCE',dest=TEMPLATE,tool=SKIDL,do_erc=True)])


import pytest
from skidl import *
from .setup_teardown import *

def test_subclass_1():

    class Resistor(Part):
        def __init__(self, value, ref=None, footprint='Resistors_SMD:R_0805'):
            super().__init__('device', 'R', value=value, ref=ref, footprint=footprint)

    gnd = Net('GND') # Ground reference.
    vin = Net('VI')  # Input voltage to the divider.
    vout = Net('VO')  # Output voltage from the divider.

    r1 = Resistor('1k')
    r2 = Resistor('500')

    r1[1] += vin  # Connect the input to the first resistor.
    r2[2] += gnd  # Connect the second resistor to ground.
    vout += r1[2], r2[1]  # Output comes from the connection of the two resistors.

    ERC()
    generate_netlist()



# MIT license
#
# Copyright (C) 2016 by XESS Corporation.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Convert a netlist into an equivalent SKiDL program.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import int
from future import standard_library
standard_library.install_aliases()

import re
from collections import defaultdict
from kinparse import parse_netlist


def netlist_to_skidl(netlist_src):

    tab = ' ' * 4

    def legalize(name):
        """Make a string into a legal python variable name."""
        return re.sub('[^a-zA-Z0-9_]', '_', name)

    def comp_key(comp):
        """Create an ID key from component's major characteristics."""
        lib = comp.libsource.lib.val
        part = comp.libsource.part.val
        # Do not include the value in the key otherwise every capacitor or
        # resistor value will get its own template.
        try:
            footprint = comp.footprint.val
            return legalize('_'.join([lib, part, footprint]))
        except AttributeError:
            # Component has no assigned footprint.
            return legalize('_'.join([lib, part]))

    def template_comp_to_skidl(template_comp):
        """Instantiate a component that will be used as a template."""
        ltab = tab

        # Instantiate component template.
        name = comp_key(template_comp)  # python variable name for template.
        lib = template_comp.libsource.lib.val
        part = template_comp.libsource.part.val
        try:
            footprint = template_comp.footprint.val
            template_comp_skidl = "{ltab}{name} = Part('{lib}', '{part}', footprint='{footprint}', dest=TEMPLATE)\n".format(
                **locals())
        except AttributeError:
            template_comp_skidl = "{ltab}{name} = Part('{lib}', '{part}', dest=TEMPLATE)\n".format(
                **locals())

        # Set attributes of template using the component fields.
        for fld in template_comp.fields:
            template_comp_skidl += "{ltab}setattr({name}, '{fld.name.val}', '{fld.text}')\n".format(
                **locals())

        return template_comp_skidl

    def comp_to_skidl(comp, template_comps):
        """Instantiate components using the component templates."""
        ltab = tab

        # Use the component key to get the template that matches this component.
        template_comp_name = comp_key(comp)
        template_comp = template_comps[template_comp_name]

        # Get the fields for the template.
        template_comp_fields = {
            fld.name.val: fld.text
            for fld in template_comp.fields
        }

        # Create a legal python variable for storing the instantiated component.
        ref = comp.ref.val
        legal_ref = legalize(ref)

        # Instantiate the component and its value (if any).
        try:
            comp_skidl = "{ltab}{legal_ref} = {template_comp_name}(ref='{ref}', value='{comp.value.val}')\n".format(
                **locals())
        except AttributeError:
            comp_skidl = "{ltab}{legal_ref} = {template_comp_name}(ref='{ref}')\n".format(
                **locals())

        # Set the fields of the instantiated component if they differ from the values in the template.
        for fld in comp.fields:
            if fld.text != template_comp_fields.get(fld.name.val, ''):
                comp_skidl += "{ltab}setattr({legal_ref}, '{fld.name.val}', '{fld.text}')\n".format(
                    **locals())

        return comp_skidl

    def net_to_skidl(net):
        """Instantiate the nets between components."""
        ltab = tab

        code = net.code.val  # Unique code integer for each net.
        name = legalize('net__' + code)  # Python variable name to storing net.
        net_name = net.name.val  # Original net name from netlist.

        # Instantiate the net.
        net_skidl = "{ltab}{name} = Net('{net_name}')\n".format(**locals())

        # Connect component pins to the net.
        net_skidl += "{ltab}{name} += ".format(**locals())
        comp_pins = []
        for n in net.nodes:
            comp_var = legalize(
                n.ref.val)  # Python variable storing component.
            pin_num = n.pin.val  # Pin number of component attached to net.
            comp_pins.append("{comp_var}['{pin_num}']".format(**locals()))
        net_skidl += ', '.join(comp_pins) + '\n'

        return net_skidl

    def _netlist_to_skidl(ntlst):
        """Convert a netlist into a skidl script."""

        # Sequence of operations:
        #   1. Create a template for each component having a given library, part name and footprint.
        #   2. Instantiate each component using its matching template. Also, set any attributes
        #      for the component that don't match those in the template.
        #   3. Instantiate the nets connecting the component pins.
        #   4. Call the script to instantiate the complete circuit.
        #   5. Generate the netlist for the circuit.

        ltab = tab

        section_div = '#' + '=' * 79
        section_comment = "\n\n{ltab}{section_div}\n{ltab}# {section_desc}\n{ltab}{section_div}\n\n"

        skidl = ''
        skidl += '# -*- coding: utf-8 -*-\n\n'
        skidl += 'from skidl import *\n\n\n'
        circuit_name = legalize(ntlst.design.source.val)
        skidl += 'def {circuit_name}():'.format(**locals())

        section_desc = 'Component templates.'
        skidl += section_comment.format(**locals())
        comp_templates = {comp_key(comp): comp for comp in ntlst.components}
        template_statements = sorted(
            [template_comp_to_skidl(c) for c in list(comp_templates.values())])
        skidl += '\n'.join(template_statements)

        section_desc = 'Component instantiations.'
        skidl += section_comment.format(**locals())
        comp_inst_statements = sorted(
            [comp_to_skidl(c, comp_templates) for c in ntlst.components])
        skidl += '\n'.join(comp_inst_statements)

        section_desc = 'Net interconnections between instantiated components.'
        skidl += section_comment.format(**locals())
        net_statements = sorted([net_to_skidl(n) for n in ntlst.nets])
        skidl += '\n'.join(net_statements)

        ltab = ''
        section_desc = 'Instantiate the circuit and generate the netlist.'
        skidl += section_comment.format(**locals())
        ltab = tab
        skidl += 'if __name__ == "__main__":\n'
        skidl += '{ltab}{circuit_name}()\n'.format(**locals())
        skidl += '{ltab}generate_netlist()\n'.format(**locals())

        return skidl

    return _netlist_to_skidl(parse_netlist(netlist_src))



# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2018 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Handles nets.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import super
from builtins import range
from future import standard_library
standard_library.install_aliases()

try:
    import __builtin__ as builtins
except ImportError:
    import builtins

from copy import deepcopy, copy
import collections

from .defines import *
from .utilities import *


class Net(object):
    """
    Lists of connected pins are stored as nets using this class.

    Args:
        name: A string with the name of the net. If None or '', then
            a unique net name will be assigned.
        circuit: The Circuit object this net belongs to.
        *pins_nets_buses: One or more Pin, Net, or Bus objects or
            lists/tuples of them to be connected to this net.

    Keyword Args:
        attribs: A dictionary of attributes and values to attach to
            the Net object.
    """

    def __init__(self, name=None, circuit=None, *pins_nets_buses, **attribs):
        from .Pin import Pin

        self._valid = True  # Make net valid before doing anything else.
        self.do_erc = True
        self._drive = Pin.NO_DRIVE
        self.pins = []
        self._name = None
        self.circuit = None
        self.code = None  # This is the net number used in a KiCad netlist file.

        # Set the Circuit object for the net first because setting the net name
        # requires a lookup of existing names in the circuit.
        # Add the net to the passed-in circuit or to the default circuit.
        if circuit:
            circuit += self
        else:
            builtins.default_circuit += self

        # Set the net name *after* the net is assigned to a circuit so the
        # net can be assigned a unique name that doesn't conflict with existing
        # nets names in the circuit.
        if name:
            self.name = name

        # Attach whatever pins were given.
        self.connect(pins_nets_buses)
        del self.iadd_flag  # Remove the += flag inserted by connect().

        # Attach additional attributes to the net.
        for k, v in attribs.items():
            setattr(self, k, v)

    def _traverse(self):
        """Return all the nets and pins attached to this net, including itself."""

        from .Pin import PhantomPin

        self.test_validity()
        prev_nets = set([self])
        nets = set([self])
        prev_pins = set([])
        pins = set(self.pins)
        while pins != prev_pins:

            # Add the nets attached to any unvisited pins.
            for pin in pins - prev_pins:
                # No use visiting a pin that is not connected to a net.
                if pin.is_connected():
                    nets |= set(pin.get_nets())

            # Update the set of previously visited pins.
            prev_pins = copy(pins)

            # Add the pins attached to any unvisited nets.
            for net in nets - prev_nets:
                pins |= set(net.pins)

            # Update the set of previously visited nets.
            prev_nets = copy(nets)

        # Remove any phantom pins that may have existed for tieing nets together.
        pins = set([p for p in pins if not isinstance(p, PhantomPin)])

        traversal = collections.namedtuple('Traversal', ['nets', 'pins'])
        traversal.nets = list(nets)
        traversal.pins = list(pins)
        return traversal

    def get_pins(self):
        """Return a list of pins attached to this net."""
        self.test_validity()
        return self._traverse().pins

    def get_nets(self):
        """Return a list of nets attached to this net, including this net."""
        self.test_validity()
        return self._traverse().nets

    def is_attached(self, pin_net_bus):
        """Return true if the pin, net or bus is attached to this one."""
        if isinstance(pin_net_bus, Net):
            return pin_net_bus in self.get_nets()
        if isinstance(pin_net_bus, Pin):
            return pin_net_bus.is_attached(self)
        if isinstance(pin_net_bus, Bus):
            for net in pin_net_bus[:]:
                if self.is_attached(net):
                    return True
            return False
        logger.error("Nets can't be attached to {}!".format(type(pin_net_bus)))
        raise Exception

    def is_movable(self):
        """
        Return true if the net is movable to another circuit.

        A net is movable if it's not part of a Circuit or if there are no pins
        attached to it.
        """

        from .Circuit import Circuit

        return not isinstance(self.circuit, Circuit) or not self.pins

    def copy(self, num_copies=1, circuit=None, **attribs):
        """
        Make zero or more copies of this net.

        Args:
            num_copies: Number of copies to make of this net.

        Keyword Args:
            attribs: Name/value pairs for setting attributes for the copy.

        Returns:
            A list of Net copies or a Net if num_copies==1.

        Raises:
            Exception if the requested number of copies is a non-integer or negative.

        Notes:
            An instance of a net can be copied just by calling it like so::

                n = Net('A')    # Create a net.
                n_copy = n()    # Copy the net.

            You can also use the multiplication operator to make copies::

                n = 10 * Net('A')  # Create an array of nets.
        """

        self.test_validity()

        num_copies = max(num_copies, find_num_copies(**attribs))

        # Check that a valid number of copies is requested.
        if not isinstance(num_copies, int):
            logger.error(
                "Can't make a non-integer number ({}) of copies of a net!".
                format(num_copies))
            raise Exception
        if num_copies < 0:
            logger.error(
                "Can't make a negative number ({}) of copies of a net!".format(
                    num_copies))
            raise Exception

        # Can't make a distinct copy of a net which already has pins on it
        # because what happens if a pin is connected to the copy? Then we have
        # to search for all the other copies to add the pin to those.
        # And what's the value of that?
        if self.pins:
            logger.error(
                "Can't make copies of a net that already has pins attached to it!"
            )
            raise Exception

        # Create a list of copies of this net.
        copies = []
        for i in range(num_copies):
            # Create a deep copy of the net.
            cpy = deepcopy(self)

            # Place the copy into either the passed-in circuit, the circuit of
            # the source net, or the default circuit.
            cpy.circuit = None
            if circuit:
                circuit += cpy
            elif self.circuit:
                self.circuit += cpy
            else:
                builtins.default_circuit += cpy

            # Add other attributes to the net copy.
            for k, v in attribs.items():
                if isinstance(v, (list, tuple)):
                    try:
                        v = v[i]
                    except IndexError:
                        logger.error(
                            "{} copies of net {} were requested, but too few elements in attribute {}!".
                            format(num_copies, self.name, k))
                        raise Exception
                setattr(cpy, k, v)

            # Place the copy into the list of copies.
            copies.append(cpy)

        return list_or_scalar(copies)

    # Make copies with the multiplication operator or by calling the object.
    __mul__ = copy
    __rmul__ = copy
    __call__ = copy

    def is_implicit(self, net_name=None):
        """Return true if the net name is implicit."""

        from .defines import NET_PREFIX

        self.test_validity()
        if net_name:
            return re.match(re.escape(NET_PREFIX), net_name)
        if self.name:
            return re.match(re.escape(NET_PREFIX), self.name)
        return True

    def connect(self, *pins_nets_buses):
        """
        Return the net after connecting other pins, nets, and buses to it.

        Args:
            *pins_nets_buses: One or more Pin, Net, or Bus objects or
                lists/tuples of them to be connected to this net.

        Returns:
            The updated net with the new connections.

        Notes:
            Connections to nets can also be made using the += operator like so::

                atmega = Part('atmel', 'ATMEGA16U2')
                net = Net()
                net += atmega[1]  # Connects pin 1 of chip to the net.
        """

        from .Pin import Pin, PhantomPin

        def merge(net):
            """
            Merge pins on net with self and then delete net.

            Args:
                net: The net to merge with self.
            """

            if isinstance(self, NCNet):
                logger.error("Can't merge with a no-connect net {}!".format(
                    self.name))
                raise Exception

            if isinstance(net, NCNet):
                logger.error("Can't merge with a no-connect net {}!".format(
                    net.name))
                raise Exception

            # No need to do anything if merging a net with itself.
            if self == net:
                return

            # If this net has pins, just attach the other net to one of them.
            if self.pins:
                self.pins[0].nets.append(net)
                net.pins.append(self.pins[0])
            # If the other net has pins, attach this net to a pin on the other net.
            elif net.pins:
                net.pins[0].nets.append(self)
                self.pins.append(net.pins[0])
            # If neither net has any pins, then attach a phantom pin to one net
            # and then connect the nets together.
            else:
                p = PhantomPin()
                connect_pin(p)
                self.pins[0].nets.append(net)
                net.pins.append(self.pins[0])

            # Update the drive of the merged nets.
            self.drive = net.drive
            net.drive = self.drive

        def connect_pin(pin):
            """Connect a pin to this net."""
            if pin not in self.pins:
                if not pin.is_connected():
                    # Remove the pin from the no-connect net if it is attached to it.
                    pin.disconnect()
                self.pins.append(pin)
                pin.nets.append(self)
            return

        self.test_validity()

        # Go through all the pins and/or nets and connect them to this net.
        for pn in expand_buses(flatten(pins_nets_buses)):
            if isinstance(pn, Net):
                if pn.circuit == self.circuit:
                    merge(pn)
                else:
                    logger.error(
                        "Can't attach nets in different circuits ({}, {})!".
                        format(pn.circuit.name, self.circuit.name))
                    raise Exception
            elif isinstance(pn, Pin):
                if not pn.part or pn.part.circuit == self.circuit:
                    if not pn.part:
                        logger.warning(
                            "Attaching non-part Pin {} to a Net {}.".format(
                                pn.name, self.name))
                    connect_pin(pn)
                else:
                    logger.error(
                        "Can't attach a part to a net in different circuits ({}, {})!".
                        format(pn.part.circuit.name, self.circuit.name))
                    raise Exception
            else:
                logger.error(
                    'Cannot attach non-Pin/non-Net {} to Net {}.'.format(
                        type(pn), self.name))
                raise Exception

        def select_name(nets):
            """Return the net with the best name among a list of nets."""

            if len(nets) == 0:
                return None  # No nets, return None.
            if len(nets) == 1:
                return nets[0]  # One net, return it.
            if len(nets) == 2:
                # Two nets, return the best of them.
                name0 = getattr(nets[0], 'name')
                name1 = getattr(nets[1], 'name')
                if not name1:
                    return nets[0]
                if not name0:
                    return nets[1]
                if self.is_implicit(name1):
                    return nets[0]
                if self.is_implicit(name0):
                    return nets[1]
                logger.warning(
                    'Merging two named nets ({name0} and {name1}) into {name0}.'.
                    format(**locals()))
                return nets[0]

            # More than two nets, so bisect the list into two smaller lists and
            # recursively find the best name from each list and then return the
            # best name of those two.
            mid_point = len(nets) // 2
            return select_name([
                select_name(nets[0:mid_point]),
                select_name(nets[mid_point:])
            ])

        # Assign the same name to all the nets that are connected to this net.
        nets = self._traverse().nets
        selected_name = getattr(select_name(self._traverse().nets), 'name')
        for net in nets:
            # Assign the name directly to each net. Using the name property
            # would cause the names to be changed so they were unique.
            net._name = selected_name  # pylint: disable=protected-access

        # Add the net to the global netlist. (It won't be added again
        # if it's already there.)
        self.circuit += self

        # Set the flag to indicate this result came from the += operator.
        self.iadd_flag = True

        return self

    # Use += to connect to nets.
    __iadd__ = connect

    def disconnect(self, pin):
        """Remove the pin from this net but not any other nets it's attached to."""
        try:
            self.pins.remove(pin)
        except ValueError:
            pass

    def generate_netlist_net(self, tool=None):
        """
        Generate the net information for inclusion in a netlist.

        Args:
            tool: The format for the netlist file (e.g., KICAD).
        """

        import skidl

        if tool is None:
            tool = skidl.get_default_tool()

        self.test_validity()

        # Don't add anything to the netlist if no pins are on this net.
        if not self.get_pins():
            return

        try:
            gen_func = getattr(self, '_gen_netlist_net_{}'.format(tool))
            return gen_func()
        except AttributeError:
            logger.error(
                "Can't generate netlist in an unknown ECAD tool format ({}).".
                format(tool))
            raise Exception

    def generate_xml_net(self, tool=None):
        """
        Generate the net information for inclusion in an XML file.

        Args:
            tool: The format for the XML file (e.g., KICAD).
        """

        import skidl

        if tool is None:
            tool = skidl.get_default_tool()

        self.test_validity()

        # Don't add anything to the XML if no pins are on this net.
        if not self.get_pins():
            return

        try:
            gen_func = getattr(self, '_gen_xml_net_{}'.format(tool))
            return gen_func()
        except AttributeError:
            logger.error(
                "Can't generate XML in an unknown ECAD tool format ({}).".
                format(tool))
            raise Exception

    def erc(self):
        """
        Do electrical rules check on a net in the schematic.
        """

        from .Pin import Pin
        from .Circuit import Circuit

        def pin_conflict_chk(pin1, pin2):
            """
            Check for conflict/contention between two pins on the same net.
            """

            if not pin1.do_erc or not pin2.do_erc:
                return

            erc_result = self.circuit.erc_pin_to_pin_chk(pin1, pin2)

            # Return if the pins are compatible.
            if erc_result == Circuit.OK:
                return

            # Otherwise, generate an error or warning message.
            msg = 'Pin conflict on net {n}: {p1} <==> {p2}'.format(
                n=pin1.net.name, p1=pin1.erc_desc(), p2=pin2.erc_desc())
            if erc_result == Circuit.WARNING:
                erc_logger.warning(msg)
            else:
                erc_logger.error(msg)

        def net_drive_chk():
            """
            Check the drive level on the net to see if it is within bounds.
            """

            # Find the maximum signal driver on this net.
            net_drive = self.drive  # Start with user-set drive level.
            pins = self.get_pins()
            for p in pins:
                net_drive = max(net_drive, Pin.pin_info[p.func]['drive'])

            if net_drive <= Pin.NO_DRIVE:
                erc_logger.warning(
                    'No drivers for net {n}'.format(n=self.name))
            for p in pins:
                if Pin.pin_info[p.func]['min_rcv'] > net_drive:
                    erc_logger.warning(
                        'Insufficient drive current on net {n} for pin {p}'.
                        format(n=self.name, p=p.erc_desc()))

        self.test_validity()

        # Skip ERC check on this net if flag is cleared.
        if not self.do_erc:
            return

        # Check the number of pins attached to the net.
        pins = self.get_pins()
        num_pins = len(pins)
        if num_pins == 0:
            erc_logger.warning(
                'No pins attached to net {n}.'.format(n=self.name))
        elif num_pins == 1:
            erc_logger.warning(
                'Only one pin ({p}) attached to net {n}.'.format(
                    p=pins[0].erc_desc(), n=self.name))
        else:
            for i in range(num_pins):
                for j in range(i + 1, num_pins):
                    pin_conflict_chk(pins[i], pins[j])

        # Check to see if the net has sufficient drive.
        net_drive_chk()

    def __str__(self):
        """Return a list of the pins on this net as a string."""
        self.test_validity()
        pins = self.get_pins()
        return self.name + ': ' + ', '.join(
            [p.__str__() for p in sorted(pins, key=str)])

    __repr__ = __str__

    def __len__(self):
        """Return the number of pins attached to this net."""
        self.test_validity()
        pins = self.get_pins()
        return len(pins)

    @property
    def name(self):
        """
        Get, set and delete the name of this net.

        When setting the net name, if another net with the same name
        is found, the name for this net is adjusted to make it unique.
        """
        return self._name

    @name.setter
    def name(self, name):
        from .defines import NET_PREFIX

        self.test_validity()
        # Remove the existing name so it doesn't cause a collision if the
        # object is renamed with its existing name.
        self._name = None

        # Now name the object with the given name or some variation
        # of it that doesn't collide with anything else in the list.
        self._name = get_unique_name(self.circuit.nets, 'name', NET_PREFIX,
                                     name)

    @name.deleter
    def name(self):
        self.test_validity()
        del self._name

    @property
    def drive(self):
        """
        Get, set and delete the drive strength of this net.

        The drive strength cannot be set to a value less than its current
        value. So as pins are added to a net, the drive strength reflects the
        maximum drive value of the pins currently on the net.
        """
        self.test_validity()
        return self._drive

    @drive.setter
    def drive(self, drive):
        self.test_validity()
        self._drive = max(drive, self._drive)

    @drive.deleter
    def drive(self):
        self.test_validity()
        del self._drive

    @property
    def valid(self):
        return self._valid

    @valid.setter
    def valid(self, val):
        self.test_validity()
        self._valid = val

    def test_validity(self):
        if self.valid:
            return
        logger.error('Net {} is no longer valid. Do not use it!'.format(
            self.name))
        raise Exception


class NCNet(Net):
    """
    Lists of unconnected pins are stored using this Net subclass.

    This is a netlist subclass used for storing lists of pins which are
    explicitly specified as not being connected. This means the ERC won't
    flag these pins as floating, but no net connections for these pins
    will be placed in the netlist so there will actually be no
    connections to these pins in the physical circuit.

    Args:
        name: A string with the name of the net. If None or '', then
            a unique net name will be assigned.
        *pins_nets_buses: One or more Pin, Net, or Bus objects or
            lists/tuples of them to be connected to this net.

    Keyword Args:
        attribs: A dictionary of attributes and values to attach to
            the object.
    """

    def __init__(self, name=None, circuit=None, *pins_nets_buses, **attribs):
        from .Pin import Pin

        super(NCNet, self).__init__(
            name=name, circuit=circuit, *pins_nets_buses, **attribs)
        self._drive = Pin.NOCONNECT_DRIVE

    def generate_netlist_net(self, tool=None):
        """NO_CONNECT nets don't generate anything for netlists."""

        import skidl

        if tool is None:
            tool = skidl.get_default_tool()

        return ''

    def erc(self):
        """No need to check NO_CONNECT nets."""
        pass

    @property
    def drive(self):
        """
        Get the drive strength of this net.

        The drive strength is always NOCONNECT_DRIVE. It can't be changed.
        The drive strength cannot be deleted.
        """
        return self._drive
